@@ -5,7 +5,7 @@
 ;; Author: Shun-ichi GOTO  <gotoh@taiyo.co.jp>,
 ;;         Hideyuki SHIRAI <shirai@meadowy.org>
 ;; Created: Wed Feb 28 03:31:00 2001
-;; Version: $Revision: 1.29.2.11 $
+;; Version: $Revision: 1.53.2.1 $
 ;; Keywords: Mew, mail, w3m, WWW, hypermedia
 
 ;; This file is a part of emacs-w3m.
@@ -48,7 +48,7 @@
 ;;
 ;; (define-key mew-summary-mode-map "T" 'mew-w3m-view-inline-image)
 ;;
-;; Press "T": Display the images included its message only.
+;; Press "T":    Toggle the visibility of the images included its message only.
 ;; Press "C-uT": Display the all images included its Text/Html part."
 ;;
 ;; (4) You can use emacs-w3m to fetch and/or browse
@@ -110,7 +110,9 @@ This variable effected only XEmacs or Emacs 21."
 
 ;; Avoid bytecompile error and warnings.
 (eval-when-compile
+  (defvar mew-use-text/html)
   (unless (fboundp 'mew-current-get-fld)
+    (autoload 'mew-coding-system-p "mew")
     (autoload 'mew-current-get-fld "mew")
     (autoload 'mew-current-get-msg "mew")
     (autoload 'mew-syntax-get-entry-by-cid "mew")
@@ -132,11 +134,21 @@ This variable effected only XEmacs or Emacs 21."
   (interactive "P")
   (mew-summary-msg-or-part
    (if allimage
-       (let ((mew-w3m-auto-insert-image t)
+       (let ((mew-use-text/html t)
+	     (mew-w3m-auto-insert-image t)
 	     (mew-w3m-use-safe-url-regexp nil))
 	 (mew-summary-display 'force))
-     (let ((mew-w3m-auto-insert-image (not mew-w3m-auto-insert-image)))
-       (mew-summary-display 'force)))))
+     (save-excursion
+       (set-buffer (mew-buffer-message))
+       (let* ((image (get-text-property (point-min) 'w3m-images))
+	      (w3m-display-inline-images image)
+	      (w3m-safe-url-regexp (when mew-w3m-use-safe-url-regexp
+				     mew-w3m-safe-url-regexp)))
+	 (w3m-toggle-inline-images)
+	 (mew-elet
+	  (put-text-property (point-min) (1+ (point-min))
+			     'w3m-images (not image))
+	  (set-buffer-modified-p nil)))))))
 
 ;; processing Text/Html contents with w3m.
 (defun mew-mime-text/html-w3m (&rest args)
@@ -197,7 +209,7 @@ This variable effected only XEmacs or Emacs 21."
 			 "-o" "strict_iso2022=0")))
 	   (w3m-region begin end xref)))
 	((null cache)	;; Mew-2 + w3m, w3mmee
-	 (w3m-region begin end xref))
+	 (w3m-region begin end xref (mew-charset-guess-region begin end)))
 	(t		;; Old Mew
 	 (setq charset (or (mew-syntax-get-param params "charset")
 			   (save-excursion
@@ -212,9 +224,13 @@ This variable effected only XEmacs or Emacs 21."
 		      (progn (insert-buffer-substring cache begin end)
 			     (point))
 		      xref))))
-       (put-text-property (point-min) (1+ (point-min)) 'w3m t)))))
+       (put-text-property (point-min) (1+ (point-min)) 'w3m t)
+       (put-text-property (point-min) (1+ (point-min))
+			  'w3m-images mew-w3m-auto-insert-image)))))
 
-(defvar w3m-mew-support-cid (fboundp 'mew-syntax-get-entry-by-cid))
+
+(defvar w3m-mew-support-cid (and (boundp 'mew-version-number)
+				 (fboundp 'mew-syntax-get-entry-by-cid)))
 
 (defun mew-w3m-cid-retrieve (url &rest args)
   (let ((output-buffer (current-buffer)))
@@ -228,7 +244,9 @@ This variable effected only XEmacs or Emacs 21."
 		 (cache (mew-cache-hit fld msg 'must-hit))
 		 (syntax (mew-cache-decode-syntax cache))
 		 cidstx beg end)
-	    (setq cidstx (mew-syntax-get-entry-by-cid syntax url))
+	    (if (string< "4.0.53" mew-version-number)
+		(setq cidstx (mew-syntax-get-entry-by-cid syntax (concat "<" url ">")))
+ 	      (setq cidstx (mew-syntax-get-entry-by-cid syntax url)))
 	    (when cidstx
 	      (setq beg (mew-syntax-get-begin cidstx))
 	      (setq end (mew-syntax-get-end cidstx))
