@@ -38,31 +38,38 @@
 (require 'w3m-util)
 (require 'w3m-proc)
 (require 'w3m-image)
+(require 'w3m-favicon)
+(eval-and-compile
+  (when (featurep 'mule)
+    (require 'w3m-ccl)))
 
 ;; Functions and variables which should be defined in the other module
 ;; at run-time.
 (eval-when-compile
-  (defvar w3m-display-inline-images)
-  (defvar w3m-current-url)
   (defvar w3m-current-title)
+  (defvar w3m-current-url)
   (defvar w3m-default-coding-system)
-  (defvar w3m-history)
-  (defvar w3m-history-flat)
+  (defvar w3m-display-inline-images)
   (defvar w3m-icon-directory)
   (defvar w3m-menubar)
+  (defvar w3m-modeline-process-status-on)
+  (defvar w3m-show-graphic-icons-in-mode-line)
   (defvar w3m-toolbar)
   (defvar w3m-toolbar-buttons)
-  (defvar w3m-use-header-line)
-  (defvar w3m-work-buffer-name)
+  (defvar w3m-use-tab)
   (defvar w3m-use-tab-menubar)
-  (autoload 'w3m-setup-tab-menu "w3m-tabmenu")
+  (defvar w3m-work-buffer-name)
+  (defvar w3m-work-buffer-list)
   (autoload 'update-tab-in-gutter "gutter-items")
   (autoload 'w3m-image-type "w3m")
-  (autoload 'w3m-retrieve "w3m"))
+  (autoload 'w3m-retrieve "w3m")
+  (autoload 'w3m-setup-tab-menu "w3m-tabmenu"))
 
 ;; Dummies to shut some XEmacs variants up.
 (eval-when-compile
-  (autoload 'unicode-to-char "XEmacs-21.5-b6_and_later"))
+  (autoload 'unicode-to-char "XEmacs-21.5-b6_and_later")
+  (unless (featurep 'mule)
+    (defalias 'find-charset 'ignore)))
 
 (require 'path-util)
 (require 'poe)
@@ -542,11 +549,9 @@ as the value."
 ;;; Gutter:
 (defcustom w3m-xmas-show-current-title-in-buffer-tab
   (and (boundp 'gutter-buffers-tab-enabled)
-       (symbol-value 'gutter-buffers-tab-enabled))
-  "If non-nil, show the title string in the buffer tab.  It has no effect
-if your XEmacs does not support the gutter items.  If you turn on this
-option, it is recommended a bit that setting both the option
-`w3m-pop-up-windows' and the option `w3m-pop-up-frames' to nil."
+       gutter-buffers-tab-enabled)
+  "If non-nil, show the title strings in the buffers tab.
+It has no effect if your XEmacs does not support the gutter items."
   :group 'w3m
   :type 'boolean
   :get (lambda (symbol)
@@ -618,18 +623,257 @@ title contains non-ascii characters, show a url name by default."
     (ad-disable-advice 'format-buffers-tab-line 'around
 		       'w3m-xmas-show-current-title-in-buffer-tab))
 
+  (defun w3m-xmas-setup-tab-in-gutter ()
+    "Set up buffers tab in the gutter."
+    (set-specifier default-gutter-visible-p
+		   (and w3m-use-tab gutter-buffers-tab-enabled t)
+		   (current-buffer)))
+  (add-hook 'w3m-mode-setup-functions 'w3m-xmas-setup-tab-in-gutter)
+  (add-hook 'w3m-select-buffer-mode-hook 'w3m-xmas-setup-tab-in-gutter)
+
   (defun w3m-xmas-update-tab-in-gutter (&rest args)
     "Update the tab control in the gutter area."
-    (update-tab-in-gutter (selected-frame)))
+    (when (and w3m-use-tab gutter-buffers-tab-enabled)
+      (update-tab-in-gutter (selected-frame))))
+  (add-hook 'w3m-display-functions 'w3m-xmas-update-tab-in-gutter))
 
-  (when (symbol-value 'gutter-buffers-tab-enabled)
-    (add-hook 'w3m-display-functions 'w3m-xmas-update-tab-in-gutter)))
+;;; Graphic icons:
+(defcustom w3m-space-before-modeline-icon ""
+  "String of space character(s) to be put in front of the modeline icon.
+It may be better to use one or more spaces if you are using oblique or
+italic font in the modeline."
+  :group 'w3m
+  :type 'string)
+
+;; Glyph images to be displayed in the modeline.
+(defvar w3m-modeline-process-status-on-icon nil)
+(defvar w3m-modeline-image-status-on-icon nil)
+(defvar w3m-modeline-status-off-icon nil)
+(defvar w3m-modeline-ssl-image-status-on-icon nil)
+(defvar w3m-modeline-ssl-status-off-icon nil)
+
+(defun w3m-initialize-graphic-icons (&optional force)
+  "Make icon images which will be displayed in the modeline."
+  (interactive "P")
+  (let ((defs '((w3m-modeline-status-off-icon
+		 "state-00.xpm"
+		 w3m-modeline-status-off)
+		(w3m-modeline-image-status-on-icon
+		 "state-01.xpm"
+		 w3m-modeline-image-status-on)
+		(w3m-modeline-ssl-status-off-icon
+		 "state-10.xpm"
+		 w3m-modeline-ssl-status-off)
+		(w3m-modeline-ssl-image-status-on-icon
+		 "state-11.xpm"
+		 w3m-modeline-ssl-image-status-on)))
+	def icon file status extent keymap)
+    (while defs
+      (setq def (car defs)
+	    defs (cdr defs)
+	    icon (car def)
+	    file (expand-file-name (nth 1 def) w3m-icon-directory)
+	    status (nth 2 def))
+      (if (and w3m-show-graphic-icons-in-mode-line
+	       (device-on-window-system-p)
+	       (featurep 'xpm)
+	       (file-exists-p file))
+	  (progn
+	    (when (or force (not (symbol-value icon)))
+	      (unless extent
+		(setq extent (make-extent nil nil)
+		      keymap (make-sparse-keymap))
+		(define-key keymap 'button2
+		  (make-modeline-command-wrapper 'w3m-reload-this-page))
+		(set-extent-keymap extent keymap)
+		(set-extent-property extent 'help-echo
+				     "button2 reloads this page"))
+	      (set icon
+		   (cons extent
+			 (make-glyph
+			  (make-image-instance (vector 'xpm :file file))))))
+	    (when (stringp (symbol-value status))
+	      ;; Save the original status strings as properties.
+	      (put status 'string (symbol-value status)))
+	    (set status (list '("" w3m-space-before-modeline-icon) icon)))
+	;; Don't use graphic icons.
+	(when (get status 'string)
+	  (set status (get status 'string)))))
+    ;; Spinner.
+    (if (and w3m-show-graphic-icons-in-mode-line
+	     (device-on-window-system-p)
+	     (featurep 'gif)
+	     (file-exists-p
+	      (setq file (expand-file-name "spinner.gif"
+					   w3m-icon-directory))))
+	(progn
+	  (when (or force (not w3m-modeline-process-status-on-icon))
+	    (setq extent (make-extent nil nil)
+		  keymap (make-sparse-keymap))
+	    (define-key keymap 'button2
+	      (make-modeline-command-wrapper 'w3m-process-stop))
+	    (set-extent-keymap extent keymap)
+	    (set-extent-property extent 'help-echo
+				 "button2 kills the current process")
+	    (setq
+	     w3m-modeline-process-status-on-icon
+	     (cons
+	      extent
+	      (make-glyph
+	       (if w3m-gifsicle-program
+		   ;; XEmacs doesn't support a transparent color on gifs,
+		   ;; so we should replace the background color of the image
+		   ;; with the modeline's one.
+		   (make-image-instance
+		    (vector
+		     'gif
+		     :data
+		     (let ((coding-system-for-read 'binary)
+			   (coding-system-for-write 'binary)
+			   format-alist)
+		       (with-temp-buffer
+			 (insert-file-contents file)
+			 (goto-char (point-max))
+			 (call-process-region
+			  (point-min) (point-max) w3m-gifsicle-program t t nil
+			  "--careful" "--delay" "10" "--loopcount=forever"
+			  "--change-color" "255,255,255"
+			  (mapconcat
+			   (lambda (c) (number-to-string (% c 256)))
+			   (color-rgb-components
+			    (face-background 'modeline))
+			   ","))
+			 (buffer-string)))))
+		 (make-image-instance (vector 'gif :file file)))))))
+	  (when (stringp w3m-modeline-process-status-on)
+	    ;; Save the original status string as a property.
+	    (put 'w3m-modeline-process-status-on 'string
+		 w3m-modeline-process-status-on))
+	  (setq w3m-modeline-process-status-on
+		'(("" w3m-space-before-modeline-icon)
+		  w3m-modeline-process-status-on-icon)))
+      ;; Don't use spinner.
+      (when (get 'w3m-modeline-process-status-on 'string)
+	(setq w3m-modeline-process-status-on
+	      (get 'w3m-modeline-process-status-on 'string))))))
 
 ;;; Miscellaneous:
 (if (featurep 'mule)
     (defalias 'multibyte-string-p 'stringp)
   (defalias 'multibyte-string-p 'ignore))
 
+(if (featurep 'mule)
+    (defun w3m-mule-unicode-p ()
+      "Check the existence as charsets of mule-unicode."
+      (and (find-charset 'mule-unicode-0100-24ff)
+	   (find-charset 'mule-unicode-2500-33ff)
+	   (find-charset 'mule-unicode-e000-ffff)))
+  (defalias 'w3m-mule-unicode-p 'ignore))
+
+(if (condition-case nil
+	(progn
+	  (unless (or itimer-process itimer-timer)
+	    (itimer-driver-start))
+	  ;; Check whether there is a bug to which the difference of
+	  ;; the present time and the time when the itimer driver was
+	  ;; woken up is subtracted from the initial itimer value.
+	  (let* ((inhibit-quit t)
+		 (ctime (current-time))
+		 (itimer-timer-last-wakeup
+		  (prog1
+		      ctime
+		    (setcar ctime (1- (car ctime)))))
+		 (itimer-list nil)
+		 (itimer (start-itimer "w3m-run-at-time" 'ignore 5)))
+	    (sleep-for 0.1) ;; Accept the timeout interrupt.
+	    (prog1
+		(> (itimer-value itimer) 0)
+	      (delete-itimer itimer))))
+      (error nil))
+    (defun w3m-run-at-time (time repeat function &rest args)
+      "Emulating function run as `run-at-time'.
+TIME should be nil meaning now, or a number of seconds from now.
+Return an itimer object which can be used in either `delete-itimer'
+or `cancel-timer'."
+      (apply #'start-itimer "w3m-run-at-time"
+	     function (if time (max time 1e-9) 1e-9)
+	     repeat nil t args))
+  (defun w3m-run-at-time (time repeat function &rest args)
+    "Emulating function run as `run-at-time' in the right way.
+TIME should be nil meaning now, or a number of seconds from now.
+Return an itimer object which can be used in either `delete-itimer'
+or `cancel-timer'."
+    (let ((itimers (list nil)))
+      (setcar
+       itimers
+       (apply #'start-itimer "w3m-run-at-time"
+	      (lambda (itimers repeat function &rest args)
+		(let ((itimer (car itimers)))
+		  (if repeat
+		      (progn
+			(set-itimer-function
+			 itimer
+			 (lambda (itimer repeat function &rest args)
+			   (set-itimer-restart itimer repeat)
+			   (set-itimer-function itimer function)
+			   (set-itimer-function-arguments itimer args)
+			   (apply function args)))
+			(set-itimer-function-arguments
+			 itimer
+			 (append (list itimer repeat function) args)))
+		    (set-itimer-function
+		     itimer
+		     (lambda (itimer function &rest args)
+		       (delete-itimer itimer)
+		       (apply function args)))
+		    (set-itimer-function-arguments
+		     itimer
+		     (append (list itimer function) args)))))
+	      1e-9 (if time (max time 1e-9) 1e-9)
+	      nil t itimers repeat function args)))))
+
+(when (featurep 'mule)
+  (defun w3m-window-hscroll (&optional window)
+    "Replacement of `window-hscroll' for XEmacs-Mule.
+XEmacs does not work correctly in the display control in case buffer
+contains characters of various width.  This function does not
+necessarily solve the problem completely."
+    (let ((hs (window-hscroll window))
+	  (spos (point-at-bol))
+	  (epos (point-at-eol))
+	  (buf (window-buffer window)))
+      (save-selected-window
+	(save-excursion
+	  (set-buffer buf)
+	  (beginning-of-line)
+	  (condition-case nil
+	      (forward-char hs)
+	    (error (goto-char (point-max))))
+	  (if (< epos (point))
+	      (+ hs (- (string-width (buffer-substring spos epos))
+		       (- epos spos)))
+	    (string-width (buffer-substring spos (point))))))))
+
+  (defun w3m-current-column ()
+    "Replacement of `current-column' for XEmacs-Mule.
+XEmacs does not work correctly in the display control in case buffer
+contains characters of various width.  This function does not
+necessarily solve the problem completely."
+    (- (point) (point-at-bol)))
+
+  (defun w3m-set-window-hscroll (window columns)
+    "Replacement of `set-window-hscroll' for XEmacs-Mule.
+XEmacs does not work correctly in the display control in case buffer
+contains characters of various width.  This function does not
+necessarily solve the problem completely."
+    (save-excursion
+      (move-to-column (max columns 0))
+      (if (> columns (current-column))
+	  (set-window-hscroll window (+ (- (point-at-eol) (point-at-bol))
+					(- columns (current-column))))
+	(set-window-hscroll window (- (point) (point-at-bol))))))
+  )
+
 (provide 'w3m-xmas)
 
 ;;; w3m-xmas.el ends here
